@@ -1,5 +1,5 @@
 /* AisleRiot - menu.h
- * Copyright (C) 1998 Jonathan Blandford <jrb@mit.edu>
+ * Copyright (C) 1998, 2003 Jonathan Blandford <jrb@mit.edu>
  *
  * This game is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,11 +20,14 @@
 #ifndef MENU_H
 #define MENU_H
 
-void random_seed ();
-void undo_callback ();
-void redo_callback ();
-void help_about_callback ();
-void create_menus ();
-void install_menu_hints ();
+void random_seed ( void );
+void undo_callback ( void );
+void redo_callback ( void );
+void help_about_callback ( void );
+void create_menus ( void );
+void install_menu_hints ( GnomeApp * );
+void toolbar_hide ( void );
+void undo_set_sensitive (gboolean state);
+void redo_set_sensitive (gboolean state);
 
 #endif
