@@ -35,13 +35,16 @@
 #include "events.h"
 #include "draw.h"
 #include <gnome.h>
+#include <gconf/gconf-client.h>
 #include <libgnomeui/gnome-window-icon.h>
+#include <glade/glade.h>
 #include "slot.h"
 #include "card.h"
 #include "cscmi.h"
 #include "menu.h"
-#include "splash.h"
+#include "statistics.h"
 #include "games-clock.h"
+#include "games-gconf.h"
 
 /*
  * Global Variables
@@ -68,6 +71,7 @@ guint            n_games;
 struct dirent    **game_dents;
 gchar            *game_file = "";
 gchar            *game_name;
+gboolean         game_in_progress = FALSE;
 gboolean         game_over;
 gboolean         game_won;
 press_data_type* press_data; 
@@ -76,6 +80,7 @@ guint            x_spacing = 5;
 guint            y_spacing = 15;
 guint            x_expanded_offset = 20;
 guint            y_expanded_offset = 20;
+GConfClient      *gconf_client = NULL;
 
 #define DEFAULT_VARIATION "klondike.scm"
 #define GNOME_SESSION_BUG
@@ -98,12 +103,16 @@ gchar* game_file_to_name (const gchar* file)
  * This function assumes that xxx.scm has a HTML help file xxx.html
  * located in the gnome/help/aisleriot/../ dir (nb '_' -> '-' for DocBook).
  */
-gpointer* game_file_to_help_entry (const gchar* file)
+static gpointer* game_file_to_help_entry (const gchar* file)
 {
 #if 0
   GnomeHelpMenuEntry* entry;
-  char* p;
-  GString* help = g_string_new (g_basename(file));
+  char* p, *filename;
+  GString* help;
+
+  filename = g_path_get_basename (file);
+  help = g_string_new (filename);
+  g_free (filename);
 
   if ((p = strrchr (help->str, '.'))) 
     g_string_truncate (help, p - help->str);
@@ -116,6 +125,8 @@ gpointer* game_file_to_help_entry (const gchar* file)
 
   g_string_free(help, TRUE);
   return (gpointer *) entry;
+#else
+  return NULL;
 #endif
 }
 
@@ -123,7 +134,7 @@ void make_title ()
 {
   char *title;
 
-  title = g_strdup_printf ("AisleRiot:  %s  ( %d )", game_name, seed);
+  title = g_strdup_printf (_("AisleRiot: %s (%d)"), game_name, seed);
 
   gtk_window_set_title (GTK_WINDOW (app), title); 
 
@@ -159,7 +170,7 @@ void eval_installed_file (char *file)
                                        message);
 
     gtk_dialog_run (GTK_DIALOG(w));
-		gtk_widget_destroy(w);
+    gtk_widget_destroy(w);
     g_free (message);
     exit (1);
   }
@@ -176,19 +187,11 @@ GnomeUIInfo rules_help[] = {
 static int
 save_state (GnomeClient *client)
 {
-  const gchar *prefix = gnome_client_get_config_prefix (client);
+  gconf_client_set_string (gconf_client, "/apps/aisleriot/game_file",
+                           game_file, NULL);
+  gconf_client_set_string (gconf_client, "/apps/aisleriot/card_options",
+                           deck_options, NULL);
 
-  gnome_config_push_prefix (prefix);
-  gnome_config_set_string ("Variation/Default File", game_file);
-  gnome_config_set_string ("Deck/Options", deck_options);
-  gnome_config_pop_prefix ();
-  gnome_config_sync();
-
-#ifndef GNOME_SESSION_BUG
-  argv[0] = "rm";
-  argv[1] = gnome_config_get_real_path (prefix);
-  gnome_client_set_discard_command (client, 2, argv);
-#endif  
   return TRUE;
 }
 
@@ -197,6 +200,14 @@ void new_game (gchar* file, guint *seedp )
   SCM size;
   gint min_w, min_h;
 
+  /* If we're aborting an old game count it as a failure for
+   * statistical purposes. */
+  if (game_in_progress) {
+    update_statistics (FALSE, 0);
+  }
+  /* The game isn't actually in progress until the user makes a move. */
+  game_in_progress = FALSE;
+
   if (file && strcmp (file, game_file)) {
     gchar buf[100];
     GtkWidget *ms;
@@ -204,6 +215,11 @@ void new_game (gchar* file, guint *seedp )
 
     game_file = file;
 
+    /* Explicitly drop any dragged cards. This shouldn't happen, but it
+     * has in the past so we should be defensive. See bug 121762. */
+    /* FIXME: we haven't actually fixed the cause of this. */
+    free_press_data ();
+    
     /* Although this line slows down game switching by a noticeable amount, we
      * add it here in order to make sure all the original functions are
      * "clean". */
@@ -211,7 +227,7 @@ void new_game (gchar* file, guint *seedp )
     eval_installed_file (file);
     if(game_name) {
       g_snprintf(buf, sizeof (buf), "%s/%s", _("Help"), game_name);
-      gnome_app_remove_menus (GNOME_APP (app), buf, 1);
+/*      gnome_app_remove_menus (GNOME_APP (app), buf, 1); */
       if(rules_help[0].user_data) {
 #if 0
 	GnomeHelpMenuEntry *entry = 
@@ -223,6 +239,7 @@ void new_game (gchar* file, guint *seedp )
       }
     }
     game_name = game_file_to_name (file);
+    update_statistics_display ();
 
     if (!dont_save)
       save_state (gnome_master_client ());
@@ -250,22 +267,29 @@ void new_game (gchar* file, guint *seedp )
   g_random_set_seed(seed);
   score = 0;
   set_score();
-
-  if(surface) 
-    timer_start();
+  timer_reset ();
 
   size = gh_call0(game_data->start_game_lambda);
   gh_eval_str ("(start-game)");
+
   min_w = gh_scm2int (gh_car (size))*get_horiz_offset() + 2*get_horiz_start();
   min_h = gh_scm2int (gh_cadr (size))*get_vert_offset() + 2*get_vert_start();
-
   gtk_widget_set_size_request (playing_area, min_w, min_h);
-  if (surface)
-    refresh_screen();
 
-  game_over = FALSE;
-  make_title();
-  end_of_game_test();
+  /* It is possible for some games to not have any moves right from the
+   * start. If this happens we redeal. */
+  if (!gh_scm2bool (gh_call0 (game_data->game_over_lambda))) {
+    new_game (file, seedp);
+  } else {
+    if (surface)
+      refresh_screen();
+    
+    undo_set_sensitive (FALSE);
+    redo_set_sensitive (FALSE);
+    
+    game_over = FALSE;
+    make_title();
+  }
 }
 
 GtkWidget *score_value;
@@ -280,27 +304,8 @@ void set_score ()
 
 GtkWidget *time_value;
 
-/* void set_time () 
- *{
- *  char b [10];
- *  sprintf (b, "%3d:%02d ", game_seconds / 60, game_seconds % 60);
- *
- *  gtk_label_set(GTK_LABEL(time_value), b);
- *}
- */
-
-gint timer_cb ()
+static gint timer_cb ()
 {
-#if 0
-/*  game_seconds++;
- *  set_time();
- *  if (game_seconds > timeout) {
- *    timeout = 3600;
- *    gh_call0(game_data->timeout_lambda);
- *    end_of_game_test();
- *  }
- *  return game_seconds < 3599; *//* give up at end of one hour */
-#endif
   timeout = 3600;
   gh_call0(game_data->timeout_lambda);
   end_of_game_test();
@@ -312,32 +317,38 @@ guint timer_timeout = 0;
 void timer_start ()
 {
   if (timer_timeout)
-/*    timer_stop();
- *  game_seconds = 0;
- */
   games_clock_stop (GAMES_CLOCK (time_value));
   timeout = 3600;
-/*  set_time();
- *  timer_timeout = gtk_timeout_add (1000, (GtkFunction) (timer_cb), NULL);
- */
   games_clock_set_seconds (GAMES_CLOCK (time_value), 0);
   games_clock_start (GAMES_CLOCK (time_value));
-  timer_timeout = gtk_timeout_add (timeout * 1000, (GtkFunction) (timer_cb), NULL);
+  timer_timeout = g_timeout_add (timeout * 1000, (GSourceFunc) (timer_cb), NULL);
 }
 
 void timer_stop ()
 {
   games_clock_stop (GAMES_CLOCK (time_value));
-  gtk_timeout_remove (timer_timeout);
+  if (timer_timeout)
+    g_source_remove (timer_timeout);
   timer_timeout = 0;
 }
 
+void timer_reset (void)
+{
+  timer_stop ();
+  games_clock_set_seconds (GAMES_CLOCK (time_value), 0);
+}
+
+guint timer_get (void)
+{
+  return (guint) games_clock_get_seconds (GAMES_CLOCK (time_value));
+}
+
 /*
- * setup suff
+ * setup stuff
  */
 
 
-void create_sol_board ()
+static void create_sol_board ()
 {
   playing_area = gtk_drawing_area_new ();
   gtk_widget_set_events (playing_area, 
@@ -361,47 +372,19 @@ void create_sol_board ()
 		      GTK_SIGNAL_FUNC (button_press_event), NULL);
 }
 
-gboolean quit_app (GtkMenuItem *menuitem)
+void quit_app (GtkMenuItem *menuitem)
 {
-  GtkWidget *box;
-  gint response;
-
-  if (!game_over) {
-    box = gtk_message_dialog_new (GTK_WINDOW (app),
-            GTK_DIALOG_MODAL,
-            GTK_MESSAGE_QUESTION,
-            GTK_BUTTONS_NONE,
-            _("Are you sure you want to quit Aisleriot?"));
-    gtk_dialog_add_buttons (GTK_DIALOG (box),
-            GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
-            GTK_STOCK_QUIT, GTK_RESPONSE_ACCEPT,
-            NULL);
-
-    
-    gtk_dialog_set_default_response (GTK_DIALOG (box), GTK_RESPONSE_REJECT);
-
-    response = gtk_dialog_run (GTK_DIALOG (box));
-    gtk_widget_destroy (box);
-
-    if (response == GTK_RESPONSE_REJECT)
-        return TRUE;
+  /* If the game isn't over, then it counts as a failure. */
+  if (game_in_progress) {
+    update_statistics (FALSE, 0);
   }
-  
-  gtk_widget_destroy (app);
+
   gtk_main_quit ();
 }
 
-void create_main_window ()
+static void create_main_window ()
 {
-  /* This is the prefix used to retrieve the state when NOT restarted: */
-  const gchar* prefix = 
-    gnome_client_get_global_config_prefix (gnome_master_client ());
-
-  app = gnome_app_new ("sol", _("Aisleriot"));
-  /* Use "prefix" as the default config location ... */
-  gnome_config_push_prefix (prefix);
-  /* ... and use it for the menubar and toolbar aw well: */
-  GNOME_APP (app)->prefix = (gchar*)prefix;
+  app = gnome_app_new ("aisleriot", _("Aisleriot"));
 
   gtk_widget_realize (app);
 
@@ -411,7 +394,7 @@ void create_main_window ()
 		      GTK_SIGNAL_FUNC (configure_event), NULL);
 }
 
-void create_press_data ()
+static void create_press_data ()
 {
   GdkWindowAttr attributes;
 
@@ -423,7 +406,7 @@ void create_press_data ()
   attributes.colormap = gdk_drawable_get_colormap (GDK_DRAWABLE(playing_area->window));
   attributes.visual = gdk_drawable_get_visual (GDK_DRAWABLE(playing_area->window));
   
-  press_data = malloc(sizeof(press_data_type));
+  press_data = g_malloc0(sizeof(press_data_type));
   press_data->moving_cards = gdk_window_new(playing_area->window, &attributes,
 					    (GDK_WA_VISUAL | GDK_WA_COLORMAP));
   press_data->status = 0;
@@ -431,42 +414,32 @@ void create_press_data ()
 
 gchar* start_game;
 
-void main_prog(int argc, char *argv[])
+static void main_prog(int argc, char *argv[])
 {
   GtkWidget *score_label, *time_label, *score_box, *status_bar;
-
+  
   seed = time(NULL);
   g_random_set_seed(seed);
 
   cscm_init();
 
-  splash_update (_("Loading images..."), 0.70);
-
   create_main_window ();
 
   load_pixmaps (app, deck_options);
- 
-  splash_update (_("Dealing game..."), 0.90);
 
   create_sol_board ();
   create_menus ();
 
   score_box = gtk_hbox_new(0, FALSE);
-  score_label = gtk_label_new (_("Score: "));
+  score_label = gtk_label_new (_("Score:"));
   gtk_box_pack_start (GTK_BOX(score_box), score_label, FALSE, FALSE, 0);
   score_value = gtk_label_new ("   0");
   gtk_box_pack_start (GTK_BOX(score_box), score_value, FALSE, FALSE, 0);
-  time_label = gtk_label_new (_("Time: "));
+  time_label = gtk_label_new (_("Time:"));
   gtk_box_pack_start (GTK_BOX(score_box), time_label, FALSE, FALSE, 0);
   time_value = games_clock_new ();
   gtk_box_pack_start (GTK_BOX(score_box), time_value, FALSE, FALSE, GNOME_PAD_SMALL);
 
-  gtk_widget_show (score_label);
-  gtk_widget_show (score_value);
-  gtk_widget_show (time_label);
-  gtk_widget_show (time_value);
-  gtk_widget_show (score_box);
-
   status_bar = gnome_appbar_new (FALSE, TRUE, FALSE);
   gtk_box_pack_end (GTK_BOX(status_bar), score_box, FALSE, FALSE, 0);
   gnome_app_set_statusbar (GNOME_APP (app), status_bar);
@@ -475,41 +448,33 @@ void main_prog(int argc, char *argv[])
 
   new_game (start_game, &seed);
 
-  splash_destroy ();
+  gtk_widget_show_all (app);
+
+  if (!gconf_client_get_bool (gconf_client,
+                             "/apps/aisleriot/show_toolbar", NULL))
+    toolbar_hide();
 
-  gtk_widget_show (app);
   create_press_data ();
 
-  gtk_widget_pop_colormap ();
-
   gtk_main ();
 
   free_pixmaps();
-  gdk_drawable_unref (surface);
-  gdk_drawable_unref (press_data->moving_cards);
+  g_object_unref (surface);
+  g_object_unref (press_data->moving_cards);
 }
 
 static void
 retrieve_state (GnomeClient *client)
 {
-  const gchar *prefix = gnome_client_get_config_prefix (client);
-
-  gnome_config_push_prefix (prefix);
-  start_game = gnome_config_get_string_with_default 
-    ("Variation/Default File=" DEFAULT_VARIATION, NULL);
-  deck_options = gnome_config_get_string ("Deck/Options");
-  gnome_config_pop_prefix ();
-
-#ifdef GNOME_SESSION_BUG
-  if (strcmp (prefix, gnome_client_get_global_config_prefix (client))) {
-    gchar *prefix = gnome_config_get_real_path (prefix);
-    prefix[ strlen (prefix) - 1] = '\0';
-    unlink (prefix);
-  }
-#endif  
+  start_game = games_gconf_get_string (gconf_client,
+                                       "/apps/aisleriot/game_file",
+                                       "klondike.scm");
+  deck_options = games_gconf_get_string (gconf_client,
+                                         "/apps/aisleriot/card_options",
+                                         "beige.png bonded.png gnome.png bold-09x14.png knuth-09x10.png knuth-18x21.png knuth-21x25.png");
 }
 
-int
+static int
 is_game (const struct dirent* dent)
 {
   return (!strcmp (g_extension_pointer (dent->d_name),"scm") &&
@@ -530,28 +495,30 @@ int main (int argc, char *argv [])
   aisleriot_opts[0].descrip = N_("Variation on game rules");
   aisleriot_opts[0].argDescrip = N_("NAME");
 
-  gnome_score_init ("Aisleriot");
-
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-   gnome_program_init ("aisleriot", VERSION,
+  gnome_program_init ("aisleriot", VERSION,
  		      LIBGNOMEUI_MODULE, 
  		      argc, argv,
  		      GNOME_PARAM_POPT_TABLE, aisleriot_opts,
  		      GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
+  glade_init ();
 
-  gtk_widget_push_colormap (gdk_rgb_get_colormap ());
-
+  gconf_client = gconf_client_get_default ();
+  games_gconf_sanity_check_string (gconf_client, "/apps/aisleriot/game_file");
+  gconf_client_add_dir (gconf_client, "/apps/aisleriot", 
+			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+  gconf_client_notify_add (gconf_client, STATISTICS_KEY, 
+			   (GConfClientNotifyFunc) load_statistics,
+			   NULL, NULL, NULL);
+  load_statistics ();
+  
   gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-aisleriot.png");
-  splash_new ();
-
-  splash_update (_("Initializing scheme..."), 0.20);
-
   g_signal_connect (GTK_OBJECT (gnome_master_client ()), "save_yourself",
 		      GTK_SIGNAL_FUNC (save_state), 
-		      (gpointer) g_basename(argv[0]));
+		      (gpointer) g_path_get_basename(argv[0]));
   g_signal_connect (GTK_OBJECT (gnome_master_client ()), "die",
 		      GTK_SIGNAL_FUNC (quit_app), NULL);
 
