@@ -1,6 +1,8 @@
+/* -*- mode:C; indent-tabs-mode:t; tab-width:8; c-basic-offset:2; -*- */
+
 /* gnome-stones - main.c
  *
- * Time-stamp: <2003-06-03 22:02:25 callum>
+ * Time-stamp: <2003-07-22 07:32:46 callum>
  *
  * Copyright (C) 1998, 2003 Carsten Schaar
  *
@@ -28,6 +30,7 @@
 #include <stdlib.h>
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
+#include "main.h"
 #include "game.h"
 #include "cave.h"
 #include "object.h"
@@ -40,39 +43,6 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 
-/* You should leave 'USE_GNOME_CANVAS' undefined, because this game
-   currently doesn't support all features with gnome_canvas stuff
-   enabled.  */
-#undef USE_GNOME_CANVAS
-
-#undef USE_KEY_RELEASE
-
-
-#define APP_NAME "gnome-stones"
-
-
-/* Definitions */
-
-#define START_DELAY 3000
-#define END_DELAY 3000
-#define CURTAIN_DELAY 50
-#define COUNTDOWN_DELAY 20
-
-#define GAME_COLS  20
-#define GAME_ROWS 12
-#define GAME_SCROLL_MAX 3
-#define GAME_SCROLL_MIN 6
-
-
-#ifdef USE_KEY_RELEASE
-#define GAME_EVENTS (GDK_KEY_PRESS_MASK             |\
-		     GDK_KEY_RELEASE_MASK)
-;
-#else
-#define GAME_EVENTS (GDK_KEY_PRESS_MASK)
-#endif
-
-
 /*****************************************************************************/
 /* Used widgets. */
 
@@ -91,7 +61,8 @@ static gboolean       player_push       = FALSE;
 /* This message will be printed in the statusline, if no message was
    specified in a game file.  */
 
-static char *default_message= N_("GNOME-Stones (c) 1998 Carsten Schaar");
+static char *default_message= N_("This is GNOME Stones");
+static GConfClient *gconf_client;
 
 
 /****************************************************************************/
@@ -125,17 +96,13 @@ game_start_cb (GtkWidget *widget, gpointer data);
 static void
 show_scores_dialog (gint pos);
 
+static void
+menu_set_sensitive (gboolean state);
+
 
 /****************************************************************************/
 /* Image stuff  */
 
-static void
-my_exit (GtkWidget *widget, gpointer client_data)
-{ 
-  sound_close(); 
-  exit (1);
-}
-
 static GdkPixbuf *
 load_image_from_path (const char *relative_name)
 {
@@ -155,7 +122,7 @@ load_image_from_path (const char *relative_name)
       char       buffer[1024];
       g_snprintf (buffer, sizeof(buffer), 
 		  _("An error occured while loading the image file \"%s\".\n"
-		    "Please make sure, that GNOME-Stones is "
+		    "Please make sure, that GNOME Stones is "
 		    "correctly installed!"), relative_name);
 					    
       widget= gtk_message_dialog_new (GTK_WINDOW(app), 
@@ -171,23 +138,74 @@ load_image_from_path (const char *relative_name)
   return image;
 }
 
+static void
+draw_text_centered (GdkDrawable *pixmap, gchar *text)
+{
+  GdkGC *gc;
+  gchar *markup;
+  gchar *template = "<span size=\"%d\" weight=\"bold\" foreground=\"black\">%s</span>";
+  PangoRectangle extent;
+  gint text_size;
+  gint x,y;
+  PangoLayout *layout;
 
+  g_return_if_fail (pixmap);
+  
+  /* FIXME: This involves an ugly, ugly hack. */
+
+  gc = gstones_view->style->black_gc;
+  layout = gtk_widget_create_pango_layout (gstones_view, "");
+  pango_layout_set_alignment (layout, PANGO_ALIGN_CENTER);
+  text_size = 36*PANGO_SCALE; /* A guess. */
+  markup = g_strdup_printf (template, text_size, text);
+  pango_layout_set_markup (layout, markup, -1);
+  pango_layout_get_extents (layout, &extent, NULL);
+  /* Now correct the guess. */
+  text_size = text_size*GAME_COLS*STONE_SIZE*0.8*PANGO_SCALE/extent.width;
+  g_free (markup);
+  markup = g_strdup_printf (template, text_size, text);
+  pango_layout_set_markup (layout, markup, -1);
+  pango_layout_get_extents (layout, NULL, &extent);
+  x = (GAME_COLS*STONE_SIZE - extent.width/PANGO_SCALE)/2;
+  y = (GAME_ROWS*STONE_SIZE - extent.height/PANGO_SCALE)/2;
+  
+  gdk_draw_layout (pixmap, gc, x, y, layout);
+  g_free (markup);
+  g_object_unref (layout);
+}
 
 static void
 title_image_load (void)
 {
-  GdkPixbuf *image= load_image_from_path ("gnome-stones/title.png");
+  GdkPixbuf *image;
+  GdkGC *gc;
+  GdkPixmap *tile;
 
+  image = load_image_from_path ("gnome-stones/title-tile.png");
+  if (image != NULL) 
+    {
+      gdk_pixbuf_render_pixmap_and_mask (image, &tile, NULL, 127);
+      gdk_pixbuf_unref (image);
+    }
 
-  gdk_pixbuf_render_pixmap_and_mask (image, &title_template, NULL, 127);
-  gdk_pixbuf_render_pixmap_and_mask (image, &title_image, NULL, 127);
+  title_template = gdk_pixmap_new (gstones_view->window,
+                                   GAME_COLS * STONE_SIZE,
+                                   GAME_ROWS * STONE_SIZE, -1);
 
-  gdk_pixbuf_unref (image);
+  gc = gdk_gc_new (gstones_view->window);
+  if (tile != NULL)
+    {
+      gdk_gc_set_tile (gc, tile);
+      gdk_gc_set_fill (gc, GDK_TILED);
+      gdk_draw_rectangle (title_template, gc,
+                          TRUE, 0, 0, -1, -1);
+    }
 
-  /* Redraw the game widget.  */
-  gtk_widget_draw (gstones_view, NULL);
-  
-  
+  draw_text_centered (title_template, _("GNOME Stones"));
+
+  title_image = gdk_pixmap_new (gstones_view->window,
+                                GAME_COLS * STONE_SIZE,
+                                GAME_ROWS * STONE_SIZE, -1);
 }
 
 
@@ -255,6 +273,8 @@ game_update_image (GtkWidget *widget)
 static void
 game_update_title (void)
 {
+  GdkGC *gc;
+
   /* Replace image with template.  */
   gdk_draw_drawable (title_image,
                      gstones_view->style->black_gc, title_template,
@@ -263,60 +283,64 @@ game_update_title (void)
 
   if (game)
     {
-      static char *gametitle= N_("Game title:");
-      static char *cavename = N_("Start cave:");
+      static char *gametitle = N_("Game title");
+      static char *cavename = N_("Start cave");
 
-      guint    width, height;
+      guint width, height;
       PangoLayout *layout;
       PangoFontDescription *pfd;
-      GList   *tmp= g_list_nth (game->start_caves, start_cave);
+      GList *tmp = g_list_nth (game->start_caves, start_cave);
 
-      layout = gtk_widget_create_pango_layout(gstones_view, _(gametitle));
-      pfd = pango_font_description_from_string("fixed");
-      pango_layout_set_font_description( layout, pfd);
+      layout = gtk_widget_create_pango_layout (gstones_view, _(gametitle));
+      pfd = pango_font_description_from_string ("[sans-serif][bold][16]");
+      pango_layout_set_font_description (layout, pfd);
       
       pango_layout_get_pixel_size (layout, &width, &height);
 
-      gdk_draw_layout (title_image,
-		       gstones_view->style->black_gc,
-		       30, 
+#ifdef USE_ORIG_TITLE_SCREEN
+      gc = gstones_view->style->black_gc;
+#else
+      gc = gstones_view->style->black_gc;
+#endif
+
+      gdk_draw_layout (title_image, gc,
+                       50, 
                        GAME_ROWS*STONE_SIZE-30-height*3-height/2,
                        layout);
 
-      pango_layout_set_text( layout, 
+      pango_layout_set_text (layout,
                              game->title,
                              strlen (_(game->title)));
 
-      gdk_draw_layout (title_image,
-		       gstones_view->style->black_gc,
-		       30, 
+      gdk_draw_layout (title_image, gc,
+		       80, 
                        GAME_ROWS*STONE_SIZE-30-height*2-height/2,
                        layout);
 
-      pango_layout_set_text( layout, 
+      pango_layout_set_text (layout, 
                              _(cavename),
                              strlen (_(cavename)));
 
-      gdk_draw_layout (title_image,
-		       gstones_view->style->black_gc,
-		       30, 
+      gdk_draw_layout (title_image, gc,
+		       50, 
                        GAME_ROWS*STONE_SIZE-30-height,
                        layout);
 
-      pango_layout_set_text( layout, 
+      pango_layout_set_text (layout, 
                              tmp->data,
                              strlen (tmp->data));
 
-      gdk_draw_layout (title_image,
-		       gstones_view->style->black_gc,
-		       30, 
+      gdk_draw_layout (title_image, gc,
+		       80, 
                        GAME_ROWS*STONE_SIZE-30,
                        layout);
 
     }  
 
   /* Redraw the game widget.  */
-  gtk_widget_draw (gstones_view, NULL);
+  gtk_widget_queue_draw_area (gstones_view, 0, 0,
+                              GAME_COLS * STONE_SIZE,
+                              GAME_ROWS * STONE_SIZE);
 }
 
 
@@ -475,21 +499,23 @@ game_widget_fill (void)
 /****************************************************************************/
 /* High score enabling/disabling */
 
-void update_score_state ()
+static void 
+update_score_state (void)
 {
         gchar **names = NULL;
         gfloat *scores = NULL;
         time_t *scoretimes = NULL;
 	gint top;
 
-	top = gnome_score_get_notable(APP_NAME, NULL, &names, &scores, &scoretimes);
+	top = gnome_score_get_notable (APP_NAME, NULL, &names,
+                                       &scores, &scoretimes);
 	if (top > 0) {
-		gtk_widget_set_sensitive (game_menu[5].widget, TRUE);
+		gtk_widget_set_sensitive (game_menu[6].widget, TRUE);
 		g_strfreev(names);
 		g_free(scores);
 		g_free(scoretimes);
 	} else {
-		gtk_widget_set_sensitive (game_menu[5].widget, FALSE);
+		gtk_widget_set_sensitive (game_menu[6].widget, FALSE);
 	}
 }
 
@@ -512,7 +538,7 @@ start_cave_delay_timeout (gpointer data);
 static void
 iteration_start (GStonesCave *cave);
 
-void
+static void
 curtain_ready (ViewCurtainMode mode)
 {
   switch (mode)
@@ -525,7 +551,7 @@ curtain_ready (ViewCurtainMode mode)
 	     CURTAIN_DISPLAY_NONE;
 
 	  state= STATE_WAITING_TO_START;
-	  gtk_timeout_add (START_DELAY, start_cave_delay_timeout, curtain_cave);
+	  g_timeout_add (START_DELAY, start_cave_delay_timeout, curtain_cave);
 	}
       else
 	{
@@ -537,7 +563,7 @@ curtain_ready (ViewCurtainMode mode)
 
     case VIEW_CURTAIN_CLOSED:
       /* If the iteration has been running, we stop it now.  */
-      if (iteration_timeout) gtk_timeout_remove (iteration_timeout);
+      if (iteration_timeout) g_source_remove (iteration_timeout);
       iteration_timeout=0;
       
       /* An existing cave must be deleted.  */
@@ -577,12 +603,14 @@ curtain_ready (ViewCurtainMode mode)
    after the sequence.  Setting 'cave' to a non NULL value, lets the
    program load a new cave and start it.  */
 
-void
+static void
 curtain_start (GStonesCave *cave)
 {
   curtain_cave= cave;
   state       = STATE_CURTAIN;
+  menu_set_sensitive (cave != NULL);
 
+  
   view_set_curtain_mode (GSTONES_VIEW (gstones_view), 
 			 VIEW_CURTAIN_ANIMATE, curtain_ready);
 }
@@ -671,20 +699,20 @@ countdown_timeout_function (gpointer data)
    after the countdown.  Setting 'cavename' to a non NULL value, lets
    the program load a new cave and start it.  */
 
-void
+static void
 countdown_start (GStonesCave *cave)
 {
   /* When showing the countdown, there is no iteration in the
      background anymore.  */
-  if (iteration_timeout) gtk_timeout_remove (iteration_timeout);
+  if (iteration_timeout) g_source_remove (iteration_timeout);
   iteration_timeout=0;
 
   state    = STATE_COUNTDOWN;
   
   countdown_timeout= 
-    gtk_timeout_add (COUNTDOWN_DELAY, 
-		     (GtkFunction) countdown_timeout_function, 
-		     (gpointer) cave);
+    g_timeout_add (COUNTDOWN_DELAY, 
+		   (GSourceFunc) countdown_timeout_function, 
+		   (gpointer) cave);
 }
 
 
@@ -857,7 +885,9 @@ iteration_timeout_function (gpointer data)
 #ifdef USE_GNOME_CANVAS  
   game_update_image (gstones_view);
 #else
-  gtk_widget_draw (gstones_view, NULL);
+  gtk_widget_queue_draw_area (gstones_view, 0, 0,
+                              GAME_COLS * STONE_SIZE,
+                              GAME_ROWS * STONE_SIZE);
 #endif
   flags= cave->flags;
 
@@ -876,7 +906,7 @@ iteration_start (GStonesCave *newcave)
   status_set_cave (cave->name);
   
   iteration_timeout= 
-    gtk_timeout_add (game->frame_rate, iteration_timeout_function, cave);
+    g_timeout_add (game->frame_rate, iteration_timeout_function, cave);
 }
 
 
@@ -884,34 +914,36 @@ iteration_start (GStonesCave *newcave)
 /****************************************************************************/
 /* Additional game stuff.  */
 
-
-void
+gboolean
 load_game (const gchar *filename, guint _start_cave)
 {
-
-  char         buffer[256];
+  gchar *basename;
+  char buffer[256];
   GStonesGame *newgame;
+  gint lastcave;
 
-  g_return_if_fail (filename);
+  if (!filename)
+    return FALSE;
+
+  /* Zero length strings cause interesting problems later, so catch them now. */
+  if (*filename == '\0')
+    return FALSE;
   
+  basename = g_path_get_basename (filename);
+
   /* We don't need to load a game twice.  */
-  if (game && (strcmp (game->filename, filename) == 0))
-    {
-      /* Give feedback in statusbar.  */
-      sprintf (buffer, "Game '%s' already loaded.", filename);
-      gnome_app_flash (GNOME_APP (app), buffer);
-      return;
-    }
-  
+  if (game && (g_utf8_collate (game->filename, filename) == 0))
+    return TRUE;
+
   /* Load game.  */
   newgame= gstones_game_load (filename);
 
   if (newgame)
     {
       /* Remove all running timeouts.  */
-      if (iteration_timeout) gtk_timeout_remove (iteration_timeout);
+      if (iteration_timeout) g_source_remove (iteration_timeout);
       iteration_timeout=0;
-      if (countdown_timeout) gtk_timeout_remove (countdown_timeout);
+      if (countdown_timeout) g_source_remove (countdown_timeout);
       countdown_timeout=0;
 
       /* Set display into title mode.  */
@@ -931,16 +963,22 @@ load_game (const gchar *filename, guint _start_cave)
          automatically, when starting a game.  */
       cave      = NULL;
       game      = newgame;
-      start_cave= _start_cave;
+
+      lastcave = g_list_length (game->start_caves) - 1;
+      start_cave= CLAMP (_start_cave, 0, lastcave);
 
       /* Update the title image.  */
       game_update_title ();
       
       /* Give feedback in statusbar.  */
-      sprintf (buffer, "Game '%s' loaded.", filename);
-      gnome_app_flash (GNOME_APP (app), buffer);
-    }
+      sprintf (buffer, "Game '%s' loaded.", basename);
 
+      gnome_app_flash (GNOME_APP (app), buffer);
+    } else
+      return FALSE;
+  
+  g_free (basename);
+  return TRUE;
 }
 
 
@@ -949,7 +987,7 @@ show_scores_dialog (gint pos)
 {
   GtkWidget *dialog;
 
-  dialog = gnome_scores_display ("GNOME-Stones", APP_NAME, NULL, pos);
+  dialog = gnome_scores_display (_("GNOME Stones"), APP_NAME, NULL, pos);
   if (dialog != NULL) {
     gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW(app));
     gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
@@ -967,7 +1005,6 @@ game_start_cb (GtkWidget *widget, gpointer data)
   GList       *tmp = NULL;
   GStonesCave *cave= NULL;
   
-
   stop_title_music();
 
   if (!game) return;
@@ -979,7 +1016,7 @@ game_start_cb (GtkWidget *widget, gpointer data)
 
   tmp = g_list_nth (game->start_caves, start_cave);
   cave= gstones_cave_load (game, tmp->data);
-  
+
   if (cave)
     curtain_start (cave);
 }
@@ -1019,13 +1056,13 @@ preferences_cb (GtkWidget *widget, gpointer data)
 }
 
 
-static void
+void
 quit_cb (GtkWidget *widget, gpointer data)
 {
   preferences_save_global ();
   sound_close();
 
-  exit (0);
+  gtk_main_quit ();
 }
 
 
@@ -1043,15 +1080,18 @@ about_cb (GtkWidget *widget, gpointer data)
   GdkPixbuf *pixbuf = NULL;
   
   const gchar *authors[]= {
-    N_("Main program: Carsten Schaar <nhadcasc@fs-maphy.uni-hannover.de>"),
-    N_("Sokoban levels: Herman Mansilla"),
+    N_("Main game:"),
+    "Carsten Schaar <nhadcasc@fs-maphy.uni-hannover.de>",
+    "",
+    N_("Sokoban levels:"),
+    "Herman Mansilla",
     NULL
   };
   gchar *documenters[] = {
                           NULL
                          };
   /* Translator credits */
-  gchar *translator_credits = _("translator_credits");
+  gchar *translator_credits = _("translator-credits");
 
   if (about != NULL) {
     gtk_window_present (GTK_WINDOW(about));
@@ -1069,12 +1109,12 @@ about_cb (GtkWidget *widget, gpointer data)
 		  g_free (filename);
 	  }
   }
-  about= gnome_about_new (_("GNOME-STONES"), VERSION,
-			  "(C) 1998 Carsten Schaar",
-			  _("A game."),
+  about= gnome_about_new (_("GNOME Stones"), VERSION,
+			  "Copyright \xc2\xa9 1998-2003 Carsten Schaar",
+			  _("Mine through dirt and collect gems."),
 			  (const char **)authors,
                           (const char **)documenters,
-                          strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+                          strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
 			  pixbuf);
   
   if (pixbuf != NULL)
@@ -1124,6 +1164,15 @@ static GnomeUIInfo main_menu[]=
 };
 
 
+/*****************************************************************************/
+/* Menu sensitivity code. */
+static void menu_set_sensitive (gboolean state)
+{
+  gtk_widget_set_sensitive (game_menu[1].widget, state);
+  gtk_widget_set_sensitive (game_menu[2].widget, state);
+  gtk_widget_set_sensitive (game_menu[4].widget, state);
+}
+
 
 /*****************************************************************************/
 /* Startup sequence.  */
@@ -1245,25 +1294,17 @@ static gint startup_function (gpointer data)
   return (index != max);
 }
 
-
-/************************************************************************/
-/* exit function */
-
-/* FIXME: too many different exit/quit functions */
-
-
-void
-gstones_exit (GnomeClient *client, gpointer client_data)
-{
-  sound_close (); 
-  exit (0);
-}
-
-
 
 /*****************************************************************************/
 /* Main function.  */
 
+GConfClient *
+get_gconf_client ()
+{
+  if (!gconf_client) 
+    gconf_client = gconf_client_get_default ();
+  return gconf_client;
+}
 
 int
 main (int argc, char *argv[])
@@ -1285,14 +1326,21 @@ main (int argc, char *argv[])
        		      GNOME_PARAM_POPT_TABLE, options,
        		      GNOME_PARAM_APP_DATADIR, DATADIR,
 		      GNOME_PARAM_APP_LIBDIR, LIBDIR, NULL);
+
+  gconf_init (argc, argv, NULL);
+  gconf_client = get_gconf_client ();
+  gconf_client_add_dir (gconf_client, "/apps/gnome-stones",
+                        GCONF_CLIENT_PRELOAD_NONE, NULL);
+
   gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-stones.png");
   /* That's what a gnome application needs:  */
-  app= gnome_app_new ("gnome-stones", _("GNOME-Stones"));
-  gtk_window_set_policy  (GTK_WINDOW (app), FALSE, FALSE, TRUE);
+  app = gnome_app_new ("gnome-stones", _("GNOME Stones"));
+  gtk_window_set_resizable  (GTK_WINDOW (app), FALSE);
 
   /* ... a menu line, ... */
   gnome_app_create_menus (GNOME_APP (app), main_menu);
-
+  menu_set_sensitive (FALSE);
+  
   /* ... a toolbar, ... */
   /*  gnome_app_create_toolbar (GNOME_APP (app), toolbar); */
 
@@ -1333,9 +1381,7 @@ main (int argc, char *argv[])
   g_signal_connect (GTK_OBJECT (app), "key_release_event",
 		      GTK_SIGNAL_FUNC (game_widget_key_release_callback), 0);
   g_signal_connect (GTK_OBJECT (app), "delete_event",
-		      GTK_SIGNAL_FUNC (gstones_exit), 0);
-  g_signal_connect (GTK_OBJECT (app), "destroy_event",
-		      GTK_SIGNAL_FUNC (gtk_main_quit), 0);
+		      GTK_SIGNAL_FUNC (quit_cb), 0);
 
   joystick_set_widget (app);
 
@@ -1347,11 +1393,12 @@ main (int argc, char *argv[])
 
   session_management_init ();
 
-  gtk_idle_add (startup_function, NULL);
+  g_idle_add (startup_function, NULL);
 
   sound_init ();
 
-  play_title_music();
+  if (pref_get_sound_enabled ())
+    play_title_music();
 
   gtk_main ();
 
